import { Node, NodeSpec } from 'prosemirror-model';
import { EditorView, NodeView } from 'prosemirror-view';
import { SvelteComponent } from 'svelte';
import { EditorState, Transaction, Plugin } from 'prosemirror-state';

interface IComponentOptions<Props extends Record<string, unknown> = Record<string, unknown>> {
    target: Element | ShadowRoot;
    anchor?: Element;
    props?: Props;
    context?: Map<unknown, unknown>;
    hydrate?: boolean;
    intro?: boolean;
    $$inline?: boolean;
}
interface SvelteNodeViewControls {
    delete: () => void;
    update: (cb: (editorState: EditorState, node?: Node, getPos?: () => number) => Transaction) => void;
}
declare type Attrs = Record<string, unknown>;
declare type ComponentProps = {
    attrs?: Attrs;
    controls?: SvelteNodeViewControls;
    contentDOM?: (node: HTMLElement) => void;
    rootDOM?: (node: HTMLElement) => void;
};
declare type SvelteComponentConstructor = new (options: IComponentOptions<ComponentProps>) => SvelteComponent;
declare type Options = {
    name: string;
};
declare function createSvelteNodeView(constructor: SvelteComponentConstructor, options: Options): (node: Node, view: EditorView, getPos: () => number) => NodeView;

interface SvelteNodeviewPluginOptions {
    nodes: {
        [key: string]: SvelteComponentConstructor;
    };
}
declare const svelteNodeViewPlugin: (options: SvelteNodeviewPluginOptions) => Plugin<any>;
declare function svelteSchemaNode(tag: string, defaultData: Record<string, unknown>): NodeSpec;

export { SvelteComponentConstructor, SvelteNodeViewControls, SvelteNodeviewPluginOptions, createSvelteNodeView, svelteNodeViewPlugin, svelteSchemaNode };
