// src/compareDeep.ts
function compareDeep(a, b) {
  if (a === b)
    return true;
  if (!(a && typeof a == "object") || !(b && typeof b == "object"))
    return false;
  const array = Array.isArray(a);
  if (Array.isArray(b) != array)
    return false;
  if (array) {
    if (a.length != b.length)
      return false;
    for (let i = 0; i < a.length; i++)
      if (!compareDeep(a[i], b[i]))
        return false;
  } else {
    for (const p in a)
      if (!(p in b) || !compareDeep(a[p], b[p]))
        return false;
    for (const p in b)
      if (!(p in a))
        return false;
  }
  return true;
}

// src/nodeview.ts
import clone from "clone";
function supressMissingPropWarnings(props, fn) {
  const re = new RegExp(
    `was created with unknown prop '(${Object.keys(props).join("|")})'`
  );
  const _warn = console.warn;
  console.warn = (message, ...optionalParams) => !(typeof message === "string" && re.test(message)) && _warn(message, ...optionalParams);
  fn();
  console.warn = _warn;
}
function createSvelteNodeView(constructor, options) {
  return (node, view, getPos) => new SvelteNodeView(constructor, options, node, view, getPos);
}
var SvelteNodeView = class {
  constructor(constructor, options, node, view, getPos) {
    this.node = node;
    this.view = view;
    this.getPos = getPos;
    this.ignoreMutation = (m) => {
      if (m.type === "attributes")
        return true;
      if (this.isComponentUpdating)
        return true;
      const mutationIsInCeFalse = Array.from(
        this.dom.querySelectorAll("[contenteditable=false]")
      ).find((e) => m.target == e || e.contains(m.target));
      if (mutationIsInCeFalse)
        return true;
      return false;
    };
    var _a;
    this.node = node;
    this.view = view;
    this.getPos = getPos;
    const target = document.createElement("div");
    const props = {
      attrs: clone(node.attrs),
      contentDOM: (node2) => this.contentDOM = node2,
      rootDOM: (node2) => this.dom = node2,
      view,
      controls: {
        delete: () => this.deleteNode(),
        update: (cb) => {
          const tr = cb(this.view.state, node, getPos);
          this.view.dispatch(tr);
        }
      }
    };
    supressMissingPropWarnings(props, () => {
      this.component = new constructor({ target, props });
    });
    this.component.$$.before_update.push(
      () => this.isComponentUpdating = true
    );
    this.component.$$.after_update.push(() => {
      window.setTimeout(() => this.isComponentUpdating = false);
    });
    this.dom = (_a = this.dom) != null ? _a : target;
    this.dom.classList.add("svelte-node-view");
    this.dom.classList.add(`svelte-node-view--${options.name}`);
    const bindCallback = (attrs) => {
      if (!compareDeep(attrs, this.node.attrs)) {
        this.view.dispatch(
          this.view.state.tr.setNodeMarkup(getPos(), null, attrs)
        );
      }
    };
    const index = this.component.$$.props["attrs"];
    this.component.$$.bound[index] = bindCallback;
  }
  deleteNode() {
    const transaction = this.view.state.tr.delete(
      this.getPos(),
      this.getPos() + 1
    );
    this.view.dispatch(transaction);
  }
  update(node) {
    if (node.type === this.node.type) {
      this.component.$set({ attrs: clone(node.attrs) });
      this.node = node;
      return true;
    }
    return false;
  }
  stopEvent(e) {
    if (e instanceof DragEvent) {
      return false;
    }
    return true;
  }
};

// src/plugin.ts
import { Plugin as ProsePlugin, PluginKey } from "prosemirror-state";
function mapValues(obj, mapFn) {
  const entries = Object.entries(obj);
  const mappedEntries = entries.map(([key, val]) => [
    key,
    mapFn(val, key)
  ]);
  return Object.fromEntries(mappedEntries);
}
var svelteNodeViewPlugin = (options) => {
  const nodeViews = mapValues(
    options.nodes,
    (ctor, name) => createSvelteNodeView(ctor, { name })
  );
  return new ProsePlugin({
    key: new PluginKey("prosemirror-svelte"),
    props: { nodeViews }
  });
};
function svelteSchemaNode(tag, defaultData) {
  return {
    draggable: true,
    attrs: mapValues(defaultData, (v) => ({ default: v })),
    parseDOM: [
      {
        tag,
        getAttrs(dom) {
          return mapValues(
            defaultData,
            (_, attr) => JSON.parse(dom.getAttribute(`data-${attr}`))
          );
        }
      }
    ],
    toDOM(node) {
      const mappedEntries = Object.entries(node.attrs).map(([key, val]) => [
        `data-${key}`,
        JSON.stringify(val)
      ]);
      return [tag, Object.fromEntries(mappedEntries)];
    }
  };
}
export {
  createSvelteNodeView,
  svelteNodeViewPlugin,
  svelteSchemaNode
};
